#include <YoggieEngine.h>
#include "Scene.h"
#include "Entity.h"
using namespace entt;

namespace YoggieEngine{
	Scene::Scene()
	{
	}

	Scene::~Scene()
	{}

	Entity Scene::AddEntity(std::string name)
	{
		Entity entity = { m_registry.create(), this };

		auto& ident = entity.AddComponent<IdentifierComponent>();
		ident.name = name;
		entity.AddComponent<TransformComponent>();


		return entity;
	}

	void Scene::Start()
	{
		// Execute start functions in scripts etc....
	}

	void Scene::Update()
	{
		// Execute Update functions in scripts etc....

		// Update transforms

		auto& transforms = m_registry.view<TransformComponent>();
		transforms.each([&](auto ent, TransformComponent& transform) {

			glm::mat4 rotationX =
			glm::rotate(
				glm::mat4(1.0f),
				glm::radians(transform.Rotation.x),
				glm::vec3(1.f, 0.f, 0.0f)
			);
			glm::mat4 rotationY = 
				glm::rotate(
					glm::mat4(1.0f),
					glm::radians(transform.Rotation.y),
					glm::vec3(0.f, 1.f, 0.0f)
				);
			glm::mat4 rotationZ =
				glm::rotate(
					glm::mat4(1.0f), 
					transform.Rotation.z,
					glm::vec3(0.f, 0.f, 1.0f)
				);

			glm::mat4 rotationMatrix = rotationY * rotationX * rotationZ;
			glm::mat4 translationMatrix = glm::translate(glm::mat4(1.0f), transform.Position);
			glm::mat4 ScaleMatrix = glm::scale(glm::mat4(1.0f), transform.Scale);

			Entity entity( ent, this );
			

			if (entity.HasComponent<RelationComponent>()) 
			{
				auto& entityRelation = entity.GetComponent<RelationComponent>();
				Entity parent = entityRelation.Parent;
				TransformComponent parentTransform = parent.GetComponent<TransformComponent>();
				glm::mat4 Model = translationMatrix * rotationMatrix * ScaleMatrix;
				transform.LocalTransform = parentTransform.LocalTransform * Model;
			}
			else {
				transform.LocalTransform = translationMatrix * rotationMatrix * ScaleMatrix;

			}
			
		});

	

	}

	void Scene::FixedUpdate()
	{
		// Execute step through physics simulation step....

	}

	void Scene::Stop()
	{
		// Execute stop functions in scripts etc....
	}

}
